/***************************************************************************
	usbua100mix.h  -  A Linux Mixer/Control Panel for the Roland UA-100
                             -------------------
    begin                : June 6, 2001
    copyright            : (C) 2001 by Michael Minn
    email                : michael@michaelminn.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

***************************************************************************/

#include "usbua100mix.h"
#include "usbua100mix.xpm"


/****************************** Channel Widgets **************************************/

usbua100mix_channel::usbua100mix_channel(usbua100mix *mixer, int channel, char *name) : QWidget(mixer, name)
{
	move(channel * 60, 25);
	resize(60, 250);
	setBackgroundMode(QWidget::PaletteMid);
	this->channel = channel;

	if (channel == 0)
    {
		input_mode = new QComboBox(this);
		input_mode->move(2, 2);
		input_mode->resize(56, 23);
		input_mode->setCurrentItem(0);
		input_mode->setEditable(FALSE);
		input_mode->insertItem("Mic1");
		input_mode->insertItem("Line");
		input_mode->insertItem("M1+2");
		connect(input_mode, SIGNAL(activated(int)), this, SLOT(slotModeChange(int)));
	}
	else
	{
		name_box = new QLineEdit(this);
		name_box->move(2, 2);
		name_box->resize(56, 23);
		name_box->setText(name);
		name_box->setFrame(FALSE);
		name_box->setReadOnly(TRUE);
		name_box->setAlignment(Qt::AlignCenter);
	}

	send1 = new QSlider(0, 127, 16, 0, QSlider::Vertical, this);
	send1->move(3, 27);
	send1->resize(27, 50);
	send1->setValue(127);
	connect(send1, SIGNAL(valueChanged(int)), this, SLOT(slotSend1(int)));

	send2 = new QSlider(0, 127, 16, 0, QSlider::Vertical, this);
	send2->move(30, 27);
	send2->resize(27, 50);
	send2->setValue(127);
	connect(send2, SIGNAL(valueChanged(int)), this, SLOT(slotSend2(int)));

	record = new QPushButton(this);
	record->move(3, 80);
	record->resize(18, 25);
	record->setText("R");
	record->setToggleButton(TRUE);
	record->setBackgroundColor(QColor(0xff,0,0));
	mixer->record_button_group->insert(record);
	connect(record, SIGNAL(pressed()), this, SLOT(slotRecord()));

	output = new QPushButton(this);
	output->move(21, 80);
	output->resize(18, 25);
	output->setText("O");
	output->setToggleButton(TRUE);
	output->setBackgroundColor(QColor(0,0xff,0));
	mixer->output_button_group->insert(output);
    connect(output, SIGNAL(pressed()), this, SLOT(slotOutput()));

	mute = new QPushButton(this);
	mute->move(39, 80);
	mute->resize(18, 25);
	mute->setText("M");
	mute->setToggleButton(TRUE);
	mute->setBackgroundColor(QColor(0,0,0xff));
    connect(mute, SIGNAL(pressed()), this, SLOT(slotMute()));

	fader = new QSlider(0, 127, 16, 0, QSlider::Vertical, this);
	fader->move(5, 107);
	fader->resize(50, 141);
	fader->setValue(27);
	connect(fader, SIGNAL(valueChanged(int)), this, SLOT(slotFader(int)));
}

void usbua100mix_channel:: slotFader(int value)
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00401105 + (channel << 8), 127 - value);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405003, 127 - value);
}

void usbua100mix_channel:: slotSend1(int value)
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00401100 + (channel << 8), 127 - value);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00404007, 127 - value);
}

void usbua100mix_channel:: slotSend2(int value)
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00401102 + (channel << 8), 127 - value);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00404009, 127 - value);
}

void usbua100mix_channel::slotRecord()
{
	if (channel == 0)
	{
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405000, 4); // Uses UA-100 channel 1 which is line/mic after insertion effect
		slotModeChange(input_mode->currentItem());
	}
	else if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405000, channel);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405000, 9);
}

void usbua100mix_channel::slotOutput()
{
	if (channel < 4)
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405001, channel);
	else
		((usbua100mix*)parent())->send_ua100_sysex_message(0x00405001, 9);
}

void usbua100mix_channel::slotMute()
{
    if (channel < 4)
		if (mute->isOn())
			((usbua100mix*)parent())->send_ua100_sysex_message(0x00401106 + ((channel + 1) << 8), 0);
		else
			((usbua100mix*)parent())->send_ua100_sysex_message(0x00401106 + ((channel + 1) << 8), 1);
}

void usbua100mix_channel::slotModeChange(int mode)
{
	((usbua100mix*)parent())->send_ua100_sysex_message(0x00401000, mode);
}


/***************************** Main Mixer Widget *************************************/

usbua100mix::usbua100mix(const char *file_name)
{
	resize(600, 300);
	setMinimumSize(300, 300);

	fileMenu = new QPopupMenu();
	fileMenu->insertItem("&Open...", this, SLOT(slotFileOpen()), CTRL+Key_O, USBUA100MIX_FILE_OPEN);
	fileMenu->insertItem("&Save", this, SLOT(slotFileSave()), CTRL+Key_S, USBUA100MIX_FILE_SAVE);
	fileMenu->insertItem("Save &as...", this, SLOT(slotFileSaveAs()), 0, USBUA100MIX_FILE_SAVE_AS);
	fileMenu->insertItem("M&IDI Port...", this, SLOT(slotFileMIDIPort()), 0, USBUA100MIX_FILE_MIDI_PORT);
	fileMenu->insertItem("E&xit", this, SLOT(slotFileExit()), CTRL+Key_Q, USBUA100MIX_FILE_EXIT);
	menuBar()->insertItem("&File", fileMenu);

	helpMenu=new QPopupMenu();
	helpMenu->insertItem("About...", this, SLOT(slotHelpAbout()), 0, USBUA100MIX_HELP_ABOUT);
	menuBar()->insertItem("&Help", helpMenu);

	statusBar()->message("Ready");

	effect_button_group = new QButtonGroup(this);
	effect_button_group->setExclusive(TRUE);

	full_effect_button = new QPushButton(this);
	full_effect_button->move(302, 25);
	full_effect_button->resize(48, 23);
	full_effect_button->setText("Full");
	full_effect_button->setToggleButton(TRUE);
	full_effect_button->setOn(TRUE);
	effect_button_group->insert(full_effect_button);
	connect(full_effect_button, SIGNAL(pressed()), this, SLOT(slotFullEffect()));

	insertion_effect_button = new QPushButton(this);
	insertion_effect_button->move(352, 25);
	insertion_effect_button->resize(48, 23);
	insertion_effect_button->setText("Ins.");
	insertion_effect_button->setToggleButton(TRUE);
	effect_button_group->insert(insertion_effect_button);
	connect(insertion_effect_button, SIGNAL(pressed()), this, SLOT(slotInsertionEffect()));

	system_effect_1_button = new QPushButton(this);
	system_effect_1_button->move(402, 25);
	system_effect_1_button->resize(48, 23);
	system_effect_1_button->setText("Sys1");
	system_effect_1_button->setToggleButton(TRUE);
	effect_button_group->insert(system_effect_1_button);
	connect(system_effect_1_button, SIGNAL(pressed()), this, SLOT(slotSystemEffect1()));

	system_effect_2_button = new QPushButton(this);
	system_effect_2_button->move(452, 25);
	system_effect_2_button->resize(48, 23);
	system_effect_2_button->setText("Sys2");
	system_effect_2_button->setToggleButton(TRUE);
	effect_button_group->insert(system_effect_2_button);
	connect(system_effect_2_button, SIGNAL(pressed()), this, SLOT(slotSystemEffect2()));

	record_button_group = new QButtonGroup(this);
	record_button_group->setExclusive(TRUE);
	output_button_group = new QButtonGroup(this);
	output_button_group->setExclusive(TRUE);

	insertion_effect = new usbua100mix_effect(this);
	insertion_effect->name_box->insertItem("None");
	insertion_effect->name_box->insertItem("00: Noise Supressor");
	insertion_effect->name_box->insertItem("01: Stereo Equalizer");
	insertion_effect->name_box->insertItem("02: Spectrum");
	insertion_effect->name_box->insertItem("03: Enhancer");
	insertion_effect->name_box->insertItem("04: Humanizer");
	insertion_effect->name_box->insertItem("05: Overdrive");
	insertion_effect->name_box->insertItem("06: Distortion");
	insertion_effect->name_box->insertItem("07: Phaser");
	insertion_effect->name_box->insertItem("08: Auto Wah");
	insertion_effect->name_box->insertItem("09: Rotary");
	insertion_effect->name_box->insertItem("10: Stereo Flanger");
	insertion_effect->name_box->insertItem("11: Step Flanger");
	insertion_effect->name_box->insertItem("12: Tremelo");
	insertion_effect->name_box->insertItem("13: Auto Pan");
	insertion_effect->name_box->insertItem("14: Compressor");
	insertion_effect->name_box->insertItem("15: Limiter");
	insertion_effect->name_box->insertItem("16: Hexa Chorus");
	insertion_effect->name_box->insertItem("17: Tremelo Chorus");
	insertion_effect->name_box->insertItem("18: Stereo Chorus");
	insertion_effect->name_box->insertItem("19: Space D Chorus");
	insertion_effect->name_box->insertItem("20: 3D Chorus");
	insertion_effect->name_box->insertItem("21: Stereo Delay");
	insertion_effect->name_box->insertItem("22: Modulation Delay");
	insertion_effect->name_box->insertItem("23: 3 Tap Delay");
	insertion_effect->name_box->insertItem("24: 4 Tap Delay");
	insertion_effect->name_box->insertItem("25: Time Control Delay");
	insertion_effect->name_box->insertItem("26: Reverb");
	insertion_effect->name_box->insertItem("27: Gate Reverb");
	insertion_effect->name_box->insertItem("28: 3D Delay");
	insertion_effect->name_box->insertItem("29: 2-voice Pitch Shifter");
	insertion_effect->name_box->insertItem("30: Feedback Pitch Shifter");
	insertion_effect->name_box->insertItem("31: 3D Auto");
	insertion_effect->name_box->insertItem("32: 3D Manual");
	insertion_effect->name_box->insertItem("33: Lo-Fi 1");
	insertion_effect->name_box->insertItem("34: Lo-Fi 2");
	insertion_effect->name_box->insertItem("35: Overdrive/Chorus");
	insertion_effect->name_box->insertItem("36: Overdrive/Flanger");
	insertion_effect->name_box->insertItem("37: Overdrive/Delay");
	insertion_effect->name_box->insertItem("38: Distortion/Chorus");
	insertion_effect->name_box->insertItem("39: Distortion/Flanger");
	insertion_effect->name_box->insertItem("40: Distortion/Delay");
	insertion_effect->name_box->insertItem("41: Enhancer/Chorus");
	insertion_effect->name_box->insertItem("42: Enhancer/Flanger");
	insertion_effect->name_box->insertItem("43: Enhancer/Delay");
	insertion_effect->name_box->insertItem("44: Chorus/Delay");
	insertion_effect->name_box->insertItem("45: Flanger/Delay");
	insertion_effect->name_box->insertItem("46: Chorus/Flanger");
	insertion_effect->name_box->insertItem("47: Rotary Multi");
	insertion_effect->name_box->insertItem("48: Guitar Multi1");
	insertion_effect->name_box->insertItem("49: Guitar Multi2");
	insertion_effect->name_box->insertItem("50: Guitar Multi3");
	insertion_effect->name_box->insertItem("51: Clean Guitar Multi1");
	insertion_effect->name_box->insertItem("52: Clean Guitar Multi2");
	insertion_effect->name_box->insertItem("53: Bass Multi");
	insertion_effect->name_box->insertItem("54: E.Piano Multi");
	insertion_effect->name_box->insertItem("55: Keyboard Multi");
	insertion_effect->name_box->insertItem("56: Chorus/Delay");
	insertion_effect->name_box->insertItem("57: Flanger/Delay");
	insertion_effect->name_box->insertItem("58: Chorus/Flanger");
	insertion_effect->name_box->insertItem("59: Overdrive/Distortion1,2");
	insertion_effect->name_box->insertItem("60: Overdrive/Distortion/Rotary");
	insertion_effect->name_box->insertItem("61: Overdrive/Distortion/Phaser");
	insertion_effect->name_box->insertItem("62: Overdrive/Distortion/Auto-wah");
	insertion_effect->name_box->insertItem("63: Phaser/Rotary");
	insertion_effect->name_box->insertItem("64: Phaser/Auto-wah");
	connect(insertion_effect->name_box, SIGNAL(activated(int)), this, SLOT(slotChangeInsertionEffect(int)));

	system_effect_1 = new usbua100mix_effect(this, 4);
	system_effect_1->name_box->insertItem("None");
	system_effect_1->name_box->insertItem("Delay");
	system_effect_1->name_box->insertItem("Chorus");
	connect(system_effect_1->name_box, SIGNAL(activated(int)), this, SLOT(slotChangeSystemEffect1(int)));

	system_effect_2 = new usbua100mix_effect(this, 5);
	system_effect_2->name_box->insertItem("None");
	system_effect_2->name_box->insertItem("Delay");
	system_effect_2->name_box->insertItem("Reverb");
	connect(system_effect_2->name_box, SIGNAL(activated(int)), this, SLOT(slotChangeSystemEffect2(int)));

	full_effect = new usbua100mix_effect(this);
	full_effect->name_box->insertItem("None");
	full_effect->name_box->insertItem("01: High Quality Reverb");
	full_effect->name_box->insertItem("02: Microphone Simulator");
	full_effect->name_box->insertItem("03: Vocoder");
	full_effect->name_box->insertItem("04: Vocal Multi");
	full_effect->name_box->insertItem("05: Game with 3D Reverb");
	connect(full_effect->name_box, SIGNAL(activated(int)), this, SLOT(slotChangeFullEffect(int)));
	full_effect_button->setOn(TRUE);

	channels[0] = new usbua100mix_channel(this, 0, "Mic1");
	channels[1] = new usbua100mix_channel(this, 1, "Mic2");
	channels[2] = new usbua100mix_channel(this, 2, "Wav1");
	channels[3] = new usbua100mix_channel(this, 3, "Wav2");
	channels[4] = new usbua100mix_channel(this, 4, "Main");

	if (file_name && file_name[0])
	{
		setCaption(file_name);
		load_mixer_settings(file_name);
	}

	setIcon(QPixmap(usbua100mix_xpm));
	setDockMenuEnabled(FALSE);

	strcpy(midi_port_name, "/dev/midi02");
	midi_port = open(midi_port_name, O_WRONLY);
	if (midi_port < 0)
		QMessageBox::information(this, strerror(errno), "Failure opening UA-100 Control Port", QMessageBox::Ok);
}

int usbua100mix::send_ua100_sysex_message(unsigned int address, unsigned int value, int data_bytes = 1)
{
	if (midi_port < 0)
		return 0;

	int message_length = 13;
	unsigned char message[16] = { 0xF0, 0x41, 0x10, 0x00, 0x11, 0x12,   0x00, 0x00, 0x00, 0x00,   0x00,  0x00, 0x00, 0x00 };

	message[6] = (address >> 24) & 0xff;
	message[7] = (address >> 16) & 0xff;
	message[8] = (address >> 8) & 0xff;
	message[9] = address & 0xff;

	if (data_bytes == 1)
	{
		message[10] = value;
		message[11] = 128 - ((message[6] + message[7] + message[8] + message[9] + message[10]) & 0x7f);
		message[12] = 0xf7;
	}
	else
	{
		message[10] = (value >> 8) & 0xff;
		message[11] = value & 0xff;
		message[12] = 128 - ((message[6] + message[7] + message[8] + message[9] + message[10] + message[11]) & 0x7f);
		message[13] = 0xf7;
		message_length = 14;
	}

	char text[64] = "";
	sprintf(text, "%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x",
		message[0], message[1], message[2], message[3], message[4], message[5], message[6],
		message[7], message[8], message[9], message[10], message[11], message[12], message[13]);

	statusBar()->message(text);

	return write(midi_port, message, message_length);
}

int usbua100mix::save_mixer_settings(const char *file_name)
{
	FILE *file = fopen(file_name, "w");

	if (!file)
	{
		perror(file_name);
		QMessageBox::information(this, file_name, "Failure saving settings", QMessageBox::Ok);
		return 0;
	}

	fprintf(file, "[MODE]\nUA100CONTROLLER=True\n");
	if (full_effect->name_box->currentItem() > 0)
		fprintf(file, "EFFMODE=2\n");
	else
		fprintf(file, "EFFMODE=1\n"); // ???
	fprintf(file, "SUBMODE=0\n\n");

	fprintf(file, "[INPUT]\nMICLINE=%d\n", channels[0]->input_mode->currentItem());
	fprintf(file, "0_PAN=64\n1_PAN=64\n");

	for (int channel = 0; channel < 4; ++channel)
    {
		fprintf(file, "%d_SEND1=%d\n", channel, 127 - channels[channel]->send1->value());
		fprintf(file, "%d_SEND2=%d\n", channel, 127 - channels[channel]->send2->value());
		fprintf(file, "%d_SUB=100\n", channel);
	}

	for (int channel = 0; channel < 4; ++channel)
		fprintf(file, "%d_FADER=%d\n", channel, 127 - channels[channel]->fader->value());

	fprintf(file, "\n[SYSTEM]\n");
	fprintf(file, "0_RETURN=%d\n0_SUB=100\n", 127 - channels[4]->send1->value());
	fprintf(file, "1_RETURN=%d\n1_SUB=100\n", 127 - channels[4]->send2->value());

	fprintf(file, "\n[OUTPUT]\n");

	if (channels[0]->record->isOn()) // ????
		fprintf(file, "1_SELECT=0\n");
	else if (channels[1]->record->isOn())
		fprintf(file, "1_SELECT=1\n");
	else if (channels[2]->record->isOn())
		fprintf(file, "1_SELECT=2\n");
	else if (channels[3]->record->isOn())
		fprintf(file, "1_SELECT=3\n");
	else
		fprintf(file, "1_SELECT=8\n");

	if (channels[0]->output->isOn()) // ????
		fprintf(file, "2_SELECT=0\n");
	else if (channels[1]->output->isOn())
		fprintf(file, "2_SELECT=1\n");
	else if (channels[2]->output->isOn())
		fprintf(file, "2_SELECT=2\n");
	else if (channels[3]->output->isOn())
		fprintf(file, "2_SELECT=3\n");
	else
		fprintf(file, "2_SELECT=8\n");

	fprintf(file, "0_FADER=%d\n", 127 - channels[4]->fader->value()); //?????
	fprintf(file, "1_FADER=100\n");

	fprintf(file, "\n[EFFECT]\n");
	for (int path = 0; path < 6; ++path)
		if (full_effect && (full_effect->path == path))
			fprintf(file, "%d_SWITCH=1\n", path);
		else if (insertion_effect && (full_effect->path == path))
			fprintf(file, "%d_SWITCH=1\n", path);
		else
			fprintf(file, "%d_SWITCH=0\n", path);

	if (full_effect->name_box->currentItem() > 0)
	{
		fprintf(file, "\n[EFFECT%02d]\n", full_effect->path + 1);
		fprintf(file, "TYPE=%d\n", full_effect->name_box->currentItem() - 1);
		fprintf(file, "NUM=%d\n", full_effect->number_of_parameters());
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (full_effect->parameter[scan])
				fprintf(file, "VALUE_%02d=%d\n", scan, full_effect->parameter[scan]->currentItem());
	}

	if (insertion_effect->name_box->currentItem() > 0)
	{
		fprintf(file, "\n[EFFECT%02d]\n", insertion_effect->path + 1);

		if (insertion_effect->name_box->currentItem() < 6)
		{
			fprintf(file, "TYPE1=0\n"); // filter
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 1);
		}
		else if (insertion_effect->name_box->currentItem() < 8)
		{
			fprintf(file, "TYPE1=1\n"); // distortion
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 6);
		}
		else if (insertion_effect->name_box->currentItem() < 15)
		{
			fprintf(file, "TYPE1=2\n"); // modulation
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 8);
		}
		else if (insertion_effect->name_box->currentItem() < 17)
		{
			fprintf(file, "TYPE1=3\n"); // compressor
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 15);
		}
		else if (insertion_effect->name_box->currentItem() < 22)
		{
			fprintf(file, "TYPE1=4\n"); // chorus
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 17);
		}
		else if (insertion_effect->name_box->currentItem() < 30)
		{
			fprintf(file, "TYPE1=5\n"); // reverb
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 22);
		}
		else if (insertion_effect->name_box->currentItem() < 32)
		{
			fprintf(file, "TYPE1=6\n"); // pitch shift
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 30);
		}
		else if (insertion_effect->name_box->currentItem() < 36)
		{
			fprintf(file, "TYPE1=7\n"); // 2 in series
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 32);
		}
		else if (insertion_effect->name_box->currentItem() < 48)
		{
			fprintf(file, "TYPE1=8\n"); // Multi
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 36);
		}
		else if (insertion_effect->name_box->currentItem() < 57)
		{
			fprintf(file, "TYPE1=9\n"); // 3 in series
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 48);
		}
		else
		{
			fprintf(file, "TYPE1=10\n"); // 2 in parallel
			fprintf(file, "TYPE2=%d\n", insertion_effect->name_box->currentItem() - 57);
		}

		fprintf(file, "NUM=%d\n", insertion_effect->number_of_parameters());
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (insertion_effect->parameter[scan])
				fprintf(file, "VALUE_%02d=%d\n", scan, insertion_effect->parameter[scan]->currentItem());
	}

	if (system_effect_1->name_box->currentItem() > 0)
	{
		fprintf(file, "\n[EFFECT05]\n");
		fprintf(file, "TYPE=%d\n", system_effect_1->name_box->currentItem() - 1);
		fprintf(file, "NUM=%d\n", system_effect_1->number_of_parameters());
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (system_effect_1->parameter[scan])
				fprintf(file, "VALUE_%02d=%d\n", scan, system_effect_1->parameter[scan]->currentItem());
	}

	if (system_effect_2->name_box->currentItem() > 0)
	{
		fprintf(file, "\n[EFFECT06]\n");
		fprintf(file, "TYPE=%d\n", system_effect_2->name_box->currentItem() - 1);
		fprintf(file, "NUM=%d\n", system_effect_2->number_of_parameters());
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (system_effect_2->parameter[scan])
				fprintf(file, "VALUE_%02d=%d\n", scan, system_effect_2->parameter[scan]->currentItem());
	}

	fclose(file);

    return 0;
}

int usbua100mix::load_mixer_settings(const char *file_name)
{
	FILE *file = fopen(file_name, "r");
	char buffer[256];
	int effect_mode = 1; // 1=compact, 2=full
	int effect_path = 0; // 0=mic/line, 1=mic2, 2=wave1, 3=wave2, 4=sys1, 5=sys2
	int is_input = 1;
	usbua100mix_effect *current_effect = NULL;

	if (!file)
	{
		perror(file_name);
		QMessageBox::information(this, file_name, "Failure loading settings", QMessageBox::Ok);
		return 0;
	}

	channels[0]->input_mode->setCurrentItem(0);

	while (fgets(buffer, sizeof(buffer), file))
		if (strncmp(buffer, "EFFMODE=", 8) == 0)
			effect_mode = atoi(&buffer[8]);

		else if (strncmp(buffer, "MICLINE=", 8) == 0)
			channels[0]->input_mode->setCurrentItem(atoi(&buffer[8]));

		else if (strncmp(&buffer[2], "SEND1=", 6) == 0)
			channels[atoi(buffer)]->send1->setValue(127 - atoi(&buffer[8]));

		else if (strncmp(&buffer[2], "SEND2=", 6) == 0)
			channels[atoi(buffer)]->send2->setValue(127 - atoi(&buffer[8]));

		else if (strncmp(buffer, "[OUTPUT]", 8) == 0)
			is_input = 0;

		else if (is_input && (strncmp(&buffer[2], "FADER=", 6) == 0))
			channels[atoi(buffer)]->fader->setValue(127 - atoi(&buffer[8]));

		else if (strncmp(buffer, "1_FADER=", 8) == 0)
			channels[4]->fader->setValue(127 - atoi(&buffer[8]));

		else if (strncmp(buffer, "0_RETURN=", 9) == 0)
			channels[4]->send1->setValue(127 - atoi(&buffer[9]));

		else if (strncmp(buffer, "1_RETURN=", 9) == 0)
			channels[4]->send2->setValue(127 - atoi(&buffer[9]));

		else if (strncmp(buffer, "1_SELECT=", 9) == 0)
		{
			if (channels[0]->input_mode->currentItem() == 0) // 2 microphones
				switch (atoi(&buffer[9]))
				{
					case 0: channels[0]->output->setOn(1); break;
					case 1: channels[1]->output->setOn(1); break;
					case 2: channels[2]->output->setOn(1); break;
					case 3: channels[3]->output->setOn(1); break;
					default: channels[4]->output->setOn(1);
				}
			else // line
				switch (atoi(&buffer[9]))
				{
					case 0: channels[0]->output->setOn(1); break;
					case 1: channels[2]->output->setOn(1); break;
					case 2: channels[3]->output->setOn(1); break;
					default: channels[4]->output->setOn(1);
				}
		}

		else if (strncmp(buffer, "2_SELECT=", 9) == 0)
		{
			if (channels[0]->input_mode->currentItem() == 0) // 2 microphones
				switch (atoi(&buffer[9]))
				{
					case 0: channels[0]->record->setOn(1); break;
					case 1: channels[1]->record->setOn(1); break;
					case 2: channels[2]->record->setOn(1); break;
					case 3: channels[3]->record->setOn(1); break;
					default: channels[4]->record->setOn(1);
				}
			else // line
				switch (atoi(&buffer[9]))
				{
					case 0: channels[0]->record->setOn(1); break;
					case 1: channels[2]->record->setOn(1); break;
					case 2: channels[3]->record->setOn(1); break;
					default: channels[4]->record->setOn(1);
				}
		}

		else if (strncmp(buffer, "[EFFECT", 7) == 0)
		{
			effect_path = atoi(&buffer[7]);
			if (effect_mode == 2)
			{
				current_effect = full_effect;
				full_effect->path_box->setCurrentItem(effect_path - 1);
			}
			else if (effect_path < 4)
			{
				current_effect = insertion_effect;
				insertion_effect->path_box->setCurrentItem(effect_path - 1);
			}
			else if (effect_path == 4)
				current_effect = system_effect_1;
			else if (effect_path == 5)
				current_effect = system_effect_2;
		}

		else if (strncmp(buffer, "TYPE=", 5) == 0)
		{
			if (current_effect == full_effect)
				slotChangeFullEffect(atoi(&buffer[5]) + 1);
			else if (current_effect == system_effect_1)
				slotChangeSystemEffect1(atoi(&buffer[5]) + 1);
			else if (current_effect == system_effect_2)
				slotChangeSystemEffect2(atoi(&buffer[5]) + 1);
		}

		else if (strncmp(buffer, "TYPE1=", 6) == 0)
		{
			int type1 = atoi(&buffer[6]);
			fgets(buffer, sizeof(buffer), file);
			int type2 = atoi(&buffer[6]);
			current_effect = insertion_effect;

			if (type1 == 0) // filter
				slotChangeInsertionEffect(1 + type2);
			else if (type1 == 1) // distortion
				slotChangeInsertionEffect(6 + type2);
			else if (type1 == 2) // modulation
				slotChangeInsertionEffect(8 + type2);
			else if (type1 == 3) // compressor
				slotChangeInsertionEffect(15 + type2);
			else if (type1 == 4) // chorus
				slotChangeInsertionEffect(17 + type2);
			else if (type1 == 5) // reverb
				slotChangeInsertionEffect(22 + type2);
			else if (type1 == 6) // pitch shift
				slotChangeInsertionEffect(30 + type2);
			else if (type1 == 7) // 2 in series
				slotChangeInsertionEffect(32 + type2);
			else if (type1 == 8) // multi effects
				slotChangeInsertionEffect(36 + type2);
			else if (type1 == 9) // 3 in series
				slotChangeInsertionEffect(48 + type2);
			else if (type1 == 10) // 2 in parallel
				slotChangeInsertionEffect(57 + type2);
		}

		else if (strncmp(buffer, "VALUE_", 6) == 0)
		{
			int parameter_number = atoi(&buffer[6]);
			if (current_effect && current_effect->parameter[parameter_number])
				current_effect->parameter[parameter_number]->setCurrentItem(atoi(&buffer[9]));
		}

	return 0;
}

/****************************** Effect Parameter Values **************************************/

static char *usbua100mix_effect_parameter_low_eq_freq[3] =
{
	"200hz", "400hz", NULL
};

static char *usbua100mix_effect_parameter_hi_eq_freq[3] =
{
	"4khz", "8khz", NULL
};

static char *usbua100mix_effect_parameter_off_on[3] =
{
	"Off", "On", NULL
};

static char *usbua100mix_effect_parameter_overdrive_distortion[3] =
{
	"Overdrive", "Distortion", NULL
};

static char *usbua100mix_effect_parameter_chorus_flanger[3] =
{
	"Chorus", "Flanger", NULL
};

static char *usbua100mix_effect_parameter_wide_narrow[3] =
{
	"Wide", "Narrow", NULL
};

static char *usbua100mix_effect_parameter_noise_type[3] =
{
	"White", "Pink", NULL
};

static char *usbua100mix_effect_parameter_filter_type[4] =
{
	"Off", "LPF", "HPF", NULL
};

static char *usbua100mix_effect_parameter_direction[3] =
{
	"Down", "Up", NULL
};

static char *usbua100mix_effect_parameter_hum[3] =
{
	"50", "60", NULL
};

static char *usbua100mix_effect_parameter_mono[3] =
{
	"Mono", "Stereo", NULL
};

static char *usbua100mix_effect_parameter_vowel[6] =
{
	"A", "I", "U", "E", "O", NULL
};

static char *usbua100mix_effect_parameter_amp[5] =
{
	"Small", "Built-in", "2-Stack", "3-Stack", NULL
};

static char *usbua100mix_effect_parameter_q[6] =
{
	"0.5", "1.0", "2.0", "4.0", "9.0", NULL
};

static char *usbua100mix_effect_parameter_reverb[7] =
{
	"Room1", "Room2", "Stage1", "Stage2", "Hall1", "Hall2", NULL
};

static char *usbua100mix_effect_parameter_disc[5] =
{
	"LP", "EP", "SP", "RND", NULL
};

static char *usbua100mix_effect_parameter_5[7] =
{
	"0", "1", "2", "3",
	"4", "5", NULL
};

static char *usbua100mix_effect_parameter_10[12] =
{
	"0", "1", "2", "3",
	"4", "5", "6", "7",
	"8", "9", "10", NULL
};

static char *usbua100mix_effect_parameter_negative_10[12] =
{
	"-10", "-9", "-8", "-7",
	"-6", "-5", "-4", "-3",
	"-2", "-1", "0", NULL
};

static char *usbua100mix_effect_parameter_plus_minus_12db[78] =
{
	"--", "--", "--", "--", "--", "--", "--", "--",
	"--", "--", "--", "--", "--", "--", "--", "--",
	"--", "--", "--", "--", "--", "--", "--", "--",
	"--", "--", "--", "--", "--", "--", "--", "--",
	"--", "--", "--", "--", "--", "--", "--", "--",
	"--", "--", "--", "--", "--", "--", "--", "--",
	"--", "--", "--", "--", "-12db", "-11db", "-10db", "-9db",
	"-8db", "-7db", "-6db", "-5db", "-4db", "-3db", "-2db", "-1db",
	"0db", "1db", "2db", "3db", "4db", "5db", "6db", "7db",
	"8db", "9db", "10db", "11db", "12db", NULL
};

static char *usbua100mix_effect_parameter_percent[129] =
{
	"0%", "0%", "1%", "2%", "3%", "3%", "4%", "5%",
	"6%", "7%", "7%", "8%", "9%", "10%", "11%", "11%",
	"12%", "13%", "14%", "14%", "15%", "16%", "17%", "18%",
	"18%", "19%", "20%", "21%", "22%", "22%", "23%", "24%",
	"25%", "25%", "26%", "27%", "28%", "29%", "29%", "30%",
	"31%", "32%", "33%", "33%", "34%", "35%", "36%", "37%",
	"37%", "38%", "39%", "40%", "40%", "41%", "42%", "43%",
	"44%", "44%", "45%", "46%", "47%", "48%", "48%", "49%",
	"50%", "51%", "51%", "52%", "53%", "54%", "55%", "55%",
	"56%", "57%", "58%", "59%", "59%", "60%", "61%", "62%",
	"62%", "63%", "64%", "65%", "66%", "66%", "67%", "68%",
	"69%", "70%", "70%", "71%", "72%", "73%", "74%", "74%",
	"75%", "76%", "77%", "77%", "78%", "79%", "80%", "81%",
	"81%", "82%", "83%", "84%", "85%", "85%", "86%", "87%",
	"88%", "88%", "89%", "90%", "91%", "92%", "92%", "93%",
	"94%", "95%", "96%", "96%", "97%", "98%", "99%", "100%", NULL
};

static char *usbua100mix_effect_parameter_pre_delay_time[129] =
{
	"0.0ms", "0.1ms", "0.2ms", "0.3ms", "0.4ms", "0.5ms", "0.6ms", "0.7ms",
	"0.8ms", "0.9ms", "1.0ms", "1.1ms", "1.2ms", "1.3ms", "1.4ms", "1.5ms",
	"1.6ms", "1.7ms", "1.8ms", "1.9ms", "2.0ms", "2.1ms", "2.2ms", "2.3ms",
	"2.4ms", "2.5ms", "2.6ms", "2.7ms", "2.8ms", "2.9ms", "3.0ms", "3.1ms",
	"3.2ms", "3.3ms", "3.4ms", "3.5ms", "3.6ms", "3.7ms", "3.8ms", "3.9ms",
	"4.0ms", "4.1ms", "4.2ms", "4.3ms", "4.4ms", "4.5ms", "4.6ms", "4.7ms",
	"4.8ms", "4.9ms", "5.0ms", "5.5ms", "6.0ms", "6.5ms", "7.0ms", "7.5ms",
	"8.0ms", "8.5ms", "9.0ms", "9.5ms", "10ms", "11ms", "12ms", "13ms",
	"14ms", "15ms", "16ms", "17ms", "18ms", "19ms", "20ms", "21ms",
	"22ms", "23ms", "24ms", "25ms", "26ms", "27ms", "28ms", "29ms",
	"30ms", "31ms", "32ms", "33ms", "34ms", "35ms", "36ms", "37ms",
	"38ms", "39ms", "40ms", "41ms", "42ms", "43ms", "44ms", "45ms",
	"46ms", "47ms", "48ms", "49ms", "50ms", "52ms", "54ms", "56ms",
	"58ms", "60ms", "62ms", "64ms", "66ms", "68ms", "70ms", "72ms",
	"74ms", "76ms", "78ms", "80ms", "82ms", "84ms", "86ms", "88ms",
	"90ms", "92ms", "94ms", "96ms", "98ms", "100ms", "100ms", "100ms", NULL
};

static char *usbua100mix_effect_parameter_delay1[129] =
{
	"200ms", "205ms", "210ms", "215ms", "220ms", "225ms", "230ms", "235ms",
	"240ms", "245ms", "250ms", "255ms", "260ms", "265ms", "270ms", "275ms",
	"280ms", "285ms", "290ms", "295ms", "300ms", "305ms", "310ms", "315ms",
	"320ms", "325ms", "330ms", "335ms", "340ms", "345ms", "350ms", "355ms",
	"360ms", "365ms", "370ms", "375ms", "380ms", "385ms", "390ms", "395ms",
	"400ms", "405ms", "410ms", "415ms", "420ms", "425ms", "430ms", "435ms",
	"440ms", "445ms", "450ms", "455ms", "460ms", "465ms", "470ms", "475ms",
	"480ms", "485ms", "490ms", "495ms", "500ms", "505ms", "510ms", "515ms",
	"520ms", "525ms", "530ms", "535ms", "540ms", "545ms", "550ms", "560ms",
	"570ms", "580ms", "590ms", "600ms", "610ms", "620ms", "630ms", "640ms",
	"650ms", "660ms", "670ms", "680ms", "690ms", "700ms", "710ms", "720ms",
	"730ms", "740ms", "750ms", "760ms", "770ms", "780ms", "790ms", "800ms",
	"810ms", "820ms", "830ms", "840ms", "850ms", "860ms", "870ms", "880ms",
	"890ms", "900ms", "910ms", "920ms", "930ms", "940ms", "950ms", "960ms",
	"970ms", "980ms", "990ms" "1000ms", "--", "--", "--", "--",
	"--", "--", "--", "--", "--", "--", "--", "--", NULL
};

static char *usbua100mix_effect_parameter_delay2[129] =
{
	"200ms", "205ms", "210ms", "215ms", "220ms", "225ms", "230ms", "235ms",
	"240ms", "245ms", "250ms", "255ms", "260ms", "265ms", "270ms", "275ms",
	"280ms", "285ms", "290ms", "295ms", "300ms", "305ms", "310ms", "315ms",
	"320ms", "325ms", "330ms", "335ms", "340ms", "345ms", "350ms", "355ms",
	"360ms", "365ms", "370ms", "375ms", "380ms", "385ms", "390ms", "395ms",
	"400ms", "405ms", "410ms", "415ms", "420ms", "425ms", "430ms", "435ms",
	"440ms", "445ms", "450ms", "455ms", "460ms", "465ms", "470ms", "475ms",
	"480ms", "485ms", "490ms", "495ms", "500ms", "505ms", "510ms", "515ms",
	"520ms", "525ms", "530ms", "535ms", "540ms", "545ms", "550ms", "555ms",
	"560ms", "565ms", "570ms", "575ms", "580ms", "585ms", "590ms", "595ms",
	"600ms", "610ms", "620ms", "630ms", "640ms", "650ms", "660ms", "670ms",
	"680ms", "690ms", "700ms", "710ms", "720ms", "730ms", "740ms", "750ms",
	"760ms", "770ms", "780ms", "790ms", "800ms", "810ms", "820ms", "830ms",
	"840ms", "850ms", "860ms", "870ms", "880ms", "890ms", "900ms", "910ms",
	"920ms", "930ms", "94ms", "950ms", "960ms", "970ms", "980ms", "990ms"
	"1000ms", "1000ms", "1000ms", "1000ms", "1000ms", "1000ms", "1000ms", "1000ms", NULL
};

static char *usbua100mix_effect_parameter_delay3[129] =
{
	"0.0ms", "0.1ms", "0.2ms", "0.3ms", "0.4ms", "0.5ms", "0.6ms", "0.7ms",
	"0.8ms", "0.9ms", "1.0ms", "1.1ms", "1.2ms", "1.3ms", "1.4ms", "1.5ms",
	"1.6ms", "1.7ms", "1.8ms", "1.9ms", "2.0ms", "2.1ms", "2.2ms", "2.3ms",
	"2.4ms", "2.5ms", "2.6ms", "2.7ms", "2.8ms", "2.9ms", "3.0ms", "3.1ms",
	"3.2ms", "3.3ms", "3.4ms", "3.5ms", "3.6ms", "3.7ms", "3.8ms", "3.9ms",
	"4.0ms", "4.1ms", "4.2ms", "4.3ms", "4.4ms", "4.5ms", "4.6ms", "4.7ms",
	"4.8ms", "4.9ms", "5.0ms", "5.5ms", "6.0ms", "6.5ms", "7.0ms", "7.5ms",
	"8.0ms", "8.5ms", "9.0ms", "9.5ms", "10ms", "11ms", "12ms", "13ms",
	"14ms", "15ms", "16ms", "17ms", "18ms", "19ms", "20ms", "21ms",
	"22ms", "23ms", "24ms", "25ms", "26ms", "27ms", "28ms", "29ms",
	"30ms", "31ms", "32ms", "33ms", "34ms", "35ms", "36ms", "37ms",
	"38ms", "39ms", "40ms", "50ms", "60ms", "70ms", "80ms", "90ms",
	"100ms", "110ms", "120ms", "130ms", "140ms", "150ms", "160ms", "170ms",
	"180ms", "190ms", "200ms", "210ms", "220ms", "230ms", "240ms", "250ms",
	"260ms", "270ms", "280ms", "290ms", "300ms", "320ms", "340ms", "360ms",
	"380ms", "400ms", "420ms", "440ms", "460ms", "480ms", "500ms", "500ms", NULL
};

static char *usbua100mix_effect_parameter_delay4[129] =
{
	"0ms", "5ms", "10ms", "15ms", "20ms", "25ms", "30ms", "35ms",
	"40ms", "45ms", "50ms", "55ms", "60ms", "65ms", "70ms", "75ms",
	"80ms", "85ms", "90ms", "95ms", "100ms", "105ms", "110ms", "115ms",
	"120ms", "125ms", "130ms", "135ms", "140ms", "145ms", "150ms", "155ms",
	"160ms", "165ms", "170ms", "175ms", "180ms", "185ms", "190ms", "195ms",
	"200ms", "205ms", "210ms", "215ms", "220ms", "225ms", "230ms", "235ms",
	"240ms", "245ms", "250ms", "255ms", "260ms", "265ms", "270ms", "275ms",
	"280ms", "285ms", "290ms", "295ms", "300ms", "305ms", "310ms", "315ms",
	"320ms", "325ms", "330ms", "335ms", "340ms", "345ms", "350ms", "355ms",
	"360ms", "365ms", "370ms", "375ms", "380ms", "385ms", "390ms", "395ms",
	"400ms", "405ms", "441ms", "415ms", "420ms", "425ms", "430ms", "435ms",
	"440ms", "445ms", "450ms", "455ms", "460ms", "465ms", "470ms", "475ms",
	"480ms", "485ms", "490ms", "495ms", "500ms", "505ms", "510ms", "515ms",
	"520ms", "525ms", "530ms", "535ms", "540ms", "545ms", "550ms", "555ms",
	"560ms", "565ms", "570ms", "575ms", "580ms", "585ms", "590ms", "595ms",
	"600ms", "605ms", "610ms", "615ms", "620ms", "625ms", "630ms", "635ms", NULL
};

static char *usbua100mix_effect_parameter_reverb_time[129] = // this is a faked series
{
	"0.1", "0.2", "0.3", "0.4", "0.5", "0.6", "0.7", "0.8",
	"0.9", "1.0", "1.1", "1.2", "1.3", "1.4", "1.5", "1.6",
	"1.7", "1.8", "1.9", "2.0", "2.1", "2.2", "2.3", "2.4",
	"2.5", "2.6", "2.7", "2.8", "2.9", "3.0", "3.1", "3.2",
	"3.3", "3.4", "3.5", "3.6", "3.7", "3.8", "3.9", "4.0",
	"4.2", "4.4", "4.6", "4.8", "5.0", "5.2", "5.4", "5.6",
	"5.8", "6.0", "6.2", "6.4", "6.6", "6.8", "7.0", "7.2",
	"7.4", "7.6", "7.8", "8.0", "8.2", "8.4", "8.6", "8.8",
	"9.0", "9.2", "9.4", "9.4", "9.8", "10.0", "10.2", "10.4",
	"10.6", "11.0", "11.5", "12.0", "12.5", "13.0", "13.5", "14.0",
	"14.5", "15.0", "15.5", "16.0", "16.5", "17.0", "17.5", "18.0",
	"18.5", "19.0", "19.5", "20.0", "20.5", "21.0", "21.5", "22.0",
	"22.5", "23.0", "23.5", "24.0", "24.5", "25.0", "25.5", "26.0",
	"26.5", "27.0", "27.5", "28.0", "28.5", "29.0", "29.5", "30.0",
	"30.5", "31.0", "31.5", "32.0", "32.5", "33.0", "33.5", "34.0",
	"34.5", "35.0", "35.5", "36.0", "36.5", "37.0", "37.5", "38.0", NULL
};

static char *usbua100mix_effect_parameter_rate1[129] =
{
	"0.05hz", "0.10hz", "0.15hz", "0.20hz", "0.25hz", "0.30hz", "0.35hz", "0.40hz",
	"0.45hz", "0.50hz", "0.55hz", "0.60hz", "0.65hz", "0.70hz", "0.75hz", "0.80hz",
	"0.85hz", "0.90hz", "0.95hz", "1.00hz", "1.05hz", "1.10hz", "1.15hz", "1.20hz",
	"1.25hz", "1.30hz", "1.35hz", "1.40hz", "1.45hz", "1.50hz", "1.55hz", "1.60hz",
	"1.65hz", "1.70hz", "1.75hz", "1.80hz", "1.85hz", "1.90hz", "1.95hz", "2.00hz",
	"2.05hz", "2.10hz", "2.15hz", "2.20hz", "2.25hz", "2.30hz", "2.35hz", "2.40hz",
	"2.45hz", "2.50hz", "2.55hz", "2.60hz", "2.65hz", "2.70hz", "2.75hz", "2.80hz",
	"2.85hz", "2.90hz", "2.95hz", "3.00hz", "3.05hz", "3.10hz", "3.15hz", "3.20hz",
	"3.25hz", "3.30hz", "3.35hz", "3.40hz", "3.45hz", "3.50hz", "3.55hz", "3.60hz",
	"3.65hz", "3.70hz", "3.75hz", "3.80hz", "3.85hz", "3.90hz", "3.95hz", "4.00hz",
	"4.05hz", "4.10hz", "4.15hz", "4.20hz", "4.25hz", "4.30hz", "4.35hz", "4.40hz",
	"4.45hz", "4.50hz", "4.55hz", "4.60hz", "4.65hz", "4.70hz", "4.75hz", "4.80hz",
	"4.85hz", "4.90hz", "4.95hz", "5.00hz", "5.10hz", "5.20hz", "5.30hz", "5.40hz",
	"5.50hz", "5.60hz", "5.70hz", "5.80hz", "5.90hz", "6.00hz", "6.10hz", "6.20hz",
	"6.30hz", "6.40hz", "6.50hz", "6.60hz", "6.70hz", "6.80hz", "6.90hz", "7.00hz",
	"7.50hz", "8.00hz", "8.50hz", "9.00hz", "9.50hz", "10hz", "10hz", "10hz", NULL
};

static char *usbua100mix_effect_parameter_rate2[129] =
{
	"0.05hz", "0.10hz", "0.15hz", "0.20hz", "0.25hz", "0.30hz", "0.35hz", "0.40hz",
	"0.45hz", "0.50hz", "0.55hz", "0.60hz", "0.65hz", "0.70hz", "0.75hz", "0.80hz",
	"0.85hz", "0.90hz", "0.95hz", "1.00hz", "1.05hz", "1.10hz", "1.15hz", "1.20hz",
	"1.25hz", "1.30hz", "1.35hz", "1.40hz", "1.45hz", "1.50hz", "1.55hz", "1.60hz",
	"1.65hz", "1.70hz", "1.75hz", "1.80hz", "1.85hz", "1.90hz", "1.95hz", "2.00hz",
	"2.05hz", "2.10hz", "2.15hz", "2.20hz", "2.25hz", "2.30hz", "2.35hz", "2.40hz",
	"2.45hz", "2.50hz", "2.55hz", "2.60hz", "2.65hz", "2.70hz", "2.75hz", "2.80hz",
	"2.85hz", "2.90hz", "2.95hz", "3.00hz", "3.05hz", "3.10hz", "3.15hz", "3.20hz",
	"3.25hz", "3.30hz", "3.35hz", "3.40hz", "3.45hz", "3.50hz", "3.55hz", "3.60hz",
	"3.65hz", "3.70hz", "3.75hz", "3.80hz", "3.85hz", "3.90hz", "3.95hz", "4.00hz",
	"4.05hz", "4.10hz", "4.15hz", "4.20hz", "4.25hz", "4.30hz", "4.35hz", "4.40hz",
	"4.45hz", "4.50hz", "4.55hz", "4.60hz", "4.65hz", "4.70hz", "4.75hz", "4.80hz",
	"4.85hz", "4.90hz", "4.95hz", "5.00hz", "5.05hz", "5.10hz", "5.15hz", "5.20hz",
	"5.25hz", "5.30hz", "5.35hz", "5.40hz", "5.45hz", "5.50hz", "5.55hz", "5.60hz",
	"5.65hz", "5.70hz", "5.75hz", "5.80hz", "5.85hz", "5.90hz", "5.95hz", "6.00hz",
	"6.05hz", "6.10hz", "6.15hz", "6.20hz", "6.25hz", "6.30hz", "6.35hz", "6.40hz", NULL
};

static char *usbua100mix_effect_parameter_hfdamp[129] =
{
	"315hz", "315hz", "315hz", "315hz", "315hz", "315hz", "315hz", "315hz",
	"400hz", "400hz", "400hz", "400hz", "400hz", "400hz", "400hz", "400hz",
	"500hz", "500hz", "500hz", "500hz", "500hz", "500hz", "500hz", "500hz",
	"630hz", "630hz", "630hz", "630hz", "630hz", "630hz", "630hz", "630hz",
	"800hz", "800hz", "800hz", "800hz", "800hz", "800hz", "800hz", "800hz",
	"1khz", "1khz", "1khz", "1khz", "1khz", "1khz", "1khz", "1khz",
	"1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz",
	"1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz",
	"2khz", "2khz", "2khz", "2khz", "2khz", "2khz", "2khz", "2khz",
	"2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz",
	"3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz",
	"4khz", "4khz", "4khz", "4khz", "4khz", "4khz", "4khz", "4khz",
	"5khz", "5khz", "5khz", "5khz", "5khz", "5khz", "5khz", "5khz",
	"6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz",
	"8khz", "8khz", "8khz", "8khz", "8khz", "8khz", "8khz", "8khz",
	"bypass", "bypass", "bypass", "bypass", "bypass", "bypass", "bypass", "bypass", NULL
};

static char *usbua100mix_effect_parameter_cutoff[129] =
{
	"250hz", "250hz", "250hz", "250hz", "250hz", "250hz", "250hz", "250hz",
	"315hz", "315hz", "315hz", "315hz", "315hz", "315hz", "315hz", "315hz",
	"400hz", "400hz", "400hz", "400hz", "400hz", "400hz", "400hz", "400hz",
	"500hz", "500hz", "500hz", "500hz", "500hz", "500hz", "500hz", "500hz",
	"630hz", "630hz", "630hz", "630hz", "630hz", "630hz", "630hz", "630hz",
	"800hz", "800hz", "800hz", "800hz", "800hz", "800hz", "800hz", "800hz",
	"1khz", "1khz", "1khz", "1khz", "1khz", "1khz", "1khz", "1khz",
	"1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz",
	"1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz",
	"2khz", "2khz", "2khz", "2khz", "2khz", "2khz", "2khz", "2khz",
	"2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz",
	"3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz",
	"4khz", "4khz", "4khz", "4khz", "4khz", "4khz", "4khz", "4khz",
	"5khz", "5khz", "5khz", "5khz", "5khz", "5khz", "5khz", "5khz",
	"6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz",
	"8khz", "8khz", "8khz", "8khz", "8khz", "8khz", "8khz", "8khz", NULL
};

static char *usbua100mix_effect_parameter_eq_freq[129] =
{
	"200hz", "200hz", "200hz", "200hz", "200hz", "200hz", "200hz", "200hz",
	"250hz", "250hz", "250hz", "250hz", "250hz", "250hz", "250hz", "250hz",
	"315hz", "315hz", "315hz", "315hz", "315hz", "315hz", "315hz", "315hz",
	"400hz", "400hz", "400hz", "400hz", "400hz", "400hz", "400hz", "400hz",
	"500hz", "500hz", "500hz", "500hz", "500hz", "500hz", "500hz", "500hz",
	"630hz", "630hz", "630hz", "630hz", "630hz", "630hz", "630hz", "630hz",
	"800hz", "800hz", "800hz", "800hz", "800hz", "800hz", "800hz", "800hz",
	"1khz", "1khz", "1khz", "1khz", "1khz", "1khz", "1khz", "1khz",
	"1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz",
	"1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz",
	"2khz", "2khz", "2khz", "2khz", "2khz", "2khz", "2khz", "2khz",
	"2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz",
	"3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz",
	"4khz", "4khz", "4khz", "4khz", "4khz", "4khz", "4khz", "4khz",
	"5khz", "5khz", "5khz", "5khz", "5khz", "5khz", "5khz", "5khz",
	"6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", NULL
};

static char *usbua100mix_effect_parameter_lpf[129] =
{
	"250hz", "250hz", "250hz", "250hz", "250hz", "250hz", "250hz", "250hz",
	"315hz", "315hz", "315hz", "315hz", "315hz", "315hz", "315hz", "315hz",
	"400hz", "400hz", "400hz", "400hz", "400hz", "400hz", "400hz", "400hz",
	"500hz", "500hz", "500hz", "500hz", "500hz", "500hz", "500hz", "500hz",
	"630hz", "630hz", "630hz", "630hz", "630hz", "630hz", "630hz", "630hz",
	"800hz", "800hz", "800hz", "800hz", "800hz", "800hz", "800hz", "800hz",
	"1khz", "1khz", "1khz", "1khz", "1khz", "1khz", "1khz", "1khz",
	"1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz", "1250hz",
	"1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz", "1.6khz",
	"2khz", "2khz", "2khz", "2khz", "2khz", "2khz", "2khz", "2khz",
	"2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz", "2.5khz",
	"3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz", "3150hz",
	"4khz", "4khz", "4khz", "4khz", "4khz", "4khz", "4khz", "4khz",
	"5khz", "5khz", "5khz", "5khz", "5khz", "5khz", "5khz", "5khz",
	"6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz", "6.3khz",
	"bypass", "bypass", "bypass", "bypass", "bypass", "bypass", "bypass", "bypass", NULL
};

static char *usbua100mix_effect_parameter_manual[129] =
{
	"100hz", "110hz", "120hz", "130hz", "140hz", "150hz", "160hz", "170hz",
	"180hz", "190hz", "200hz", "210hz", "220hz", "230hz", "240hz", "250hz",
	"260hz", "270hz", "280hz", "290hz", "300hz", "320hz", "340hz", "360hz",
	"380hz", "400hz", "420hz", "440hz", "460hz", "480hz", "500hz", "520hz",
	"540hz", "560hz", "580hz", "600hz", "620hz", "640hz", "660hz", "680hz",
	"700hz", "720hz", "740hz", "760hz", "780hz", "800hz", "820hz", "840hz",
	"860hz", "880hz", "900hz", "920hz", "940hz", "960hz", "980hz", "1khz",
	"1.1khz", "1.2khz", "1.3khz", "1.4khz", "1.5khz", "1.6khz", "1.7khz", "1.8khz",
	"1.9khz", "2.0khz", "2.1khz", "2.2khz", "2.3khz", "2.4khz", "2.5khz", "2.6khz",
	"2.7khz", "2.8khz", "2.9khz", "3.0khz", "3.1khz", "3.2khz", "3.3khz", "3.4khz",
	"3.5khz", "3.6khz", "3.7khz", "3.8khz", "3.9khz", "4.0khz", "4.1khz", "4.2khz",
	"4.3khz", "4.4khz", "4.5khz", "4.6khz", "4.7khz", "4.8khz", "4.9khz", "5.0khz",
	"5.1khz", "5.2khz", "5.3khz", "5.4khz", "5.5khz", "5.6khz", "5.7khz", "5.8khz",
	"5.9khz", "6.0khz", "6.1khz", "6.2khz", "6.3khz", "6.4khz", "6.5khz", "6.6khz",
	"6.7khz", "6.8khz", "6.9khz", "7.0khz", "7.1khz", "7.2khz", "7.3khz", "7.4khz",
	"7.5khz", "7.6khz", "7.7khz", "7.8khz", "7.9khz", "8.0khz", "8.0khz", "8.0khz", NULL
};

static char *usbua100mix_effect_parameter_azimuth[129] =
{
	"L180", "L180", "L180", "L180", "L180", "L180", "L168", "L168",
	"L168", "L168", "L156", "L156", "L156", "L156", "L144", "L144",
	"L144", "L144", "L132", "L132", "L132", "L132", "L120", "L120",
	"L120", "L120", "L108", "L108", "L108", "L108", "L96", "L96",
	"L96", "L96", "L84", "L84", "L84", "L84", "L72", "L72",
	"L72", "L72", "L60", "L60", "L60", "L60", "L48", "L48",
	"L48", "L48", "L36", "L36", "L36", "L36", "L24", "L24",
	"L24", "L24", "L12", "L12", "L12", "L12", "Center", "Center",
	"Center", "Center", "R12", "R12", "R12", "R12", "R24", "R24",
	"R24", "R24", "R36", "R36", "R36", "R36", "R48", "R48",
	"R48", "R48", "R60", "R60", "R60", "R60", "R72", "R72",
	"R72", "R72", "R84", "R84", "R84", "R84", "R96", "R96",
	"R96", "R96", "R108", "R108", "R108", "R108", "R120", "R120",
	"R120", "R120", "R132", "R132", "R132", "R132", "R144", "R144",
	"R144", "R144", "R156", "R156", "R156", "R156", "R168", "R168",
	"R168", "R168", "R180", "R180", "R180", "R180", "R180", "R180",
};

static char *usbua100mix_effect_parameter_pan[129] =
{
	"L63", "L62", "L61", "L60", "L59", "L58", "L57", "L56",
	"L55", "L54", "L53", "L52", "L51", "L50", "L49", "L48",
	"L47", "L46", "L45", "L44", "L43", "L42", "L41", "L40",
	"L39", "L38", "L37", "L36", "L35", "L34", "L33", "L32",
	"L31", "L30", "L29", "L28", "L27", "L26", "L25", "L24",
	"L23", "L22", "L21", "L20", "L19", "L18", "L17", "L16",
	"L15", "L14", "L13", "L12", "L11", "L10", "L9", "L8",
	"L7", "L6", "L5", "L4", "L3", "L2", "L1", "Center",
	"Center", "R1", "R2", "R3", "R4", "R5", "R6", "R7",
	"R8", "R9", "R10", "R11", "R12", "R13", "R14", "R15",
	"R16", "R17", "R18", "R19", "R20", "R21", "R22", "R23",
	"R24", "R25", "R26", "R27", "R28", "R29", "R30", "R31",
	"R32", "R33", "R34", "R35", "R36", "R37", "R38", "R39",
	"R40", "R41", "R42", "R42", "R44", "R45", "R46", "R47",
	"R48", "R49", "R50", "R51", "R52", "R53", "R54", "R55",
	"R56", "R57", "R58", "R59", "R60", "R61", "R62", "R63", NULL
};


static char *usbua100mix_effect_parameter_accel[129] =
{
	"0", "0", "0", "0", "0", "0", "0", "0",
	"1", "1", "1", "1", "1", "1", "1", "1",
	"2", "2", "2", "2", "2", "2", "2", "2",
	"3", "3", "3", "3", "3", "3", "3", "3",
	"4", "4", "4", "4", "4", "4", "4", "4",
	"5", "5", "5", "5", "5", "5", "5", "5",
	"6", "6", "6", "6", "6", "6", "6", "6",
	"7", "7", "7", "7", "7", "7", "7", "7",
	"8", "8", "8", "8", "8", "8", "8", "8",
	"9", "9", "9", "9", "9", "9", "9", "9",
	"10", "10", "10", "10", "10", "10", "10", "10",
	"11", "11", "11", "11", "11", "11", "11", "11",
	"12", "12", "12", "12", "12", "12", "12", "12",
	"13", "13", "13", "13", "13", "13", "13", "13",
	"14", "14", "14", "14", "14", "14", "14", "14",
	"15", "15", "15", "15", "15", "15", "15", "15", NULL
};

/********************** Slots ************************/

void usbua100mix::slotFileOpen()
{
	struct stat file_status;

	QString file_name = QFileDialog::getOpenFileName(0, "Mixer Settings (*.uax);;All Files (*.*)", this);

	if (file_name.isEmpty())
		return;

	else if (stat(file_name, &file_status) < 0)
		QMessageBox::information(this, strerror(errno), "Failure opening file", QMessageBox::Ok);

	else
	{
		load_mixer_settings(file_name);
		setCaption(file_name);
		statusBar()->message("New Settings");
	}
}

void usbua100mix::slotFileSave()
{
	if (caption().isEmpty())
		slotFileSaveAs();

	else
		save_mixer_settings(caption());

	statusBar()->message("Mixer settings saved");
}

void usbua100mix::slotFileSaveAs()
{
	statusBar()->message("Saving file under new filename...");

	QString file_name = QFileDialog::getSaveFileName(NULL, "Mixer settings (*.uax);;All Files (*.*)", this);

	if (!file_name.isEmpty())
	{
		if (!strstr(file_name, ".uax"))
			file_name.append(".uax");

		setCaption(file_name);
		slotFileSave();
	}
	else
		statusBar()->message("Mixer settings not saved");
}

void usbua100mix::slotFileMIDIPort()
{
	strcpy(midi_port_name, QFileDialog::getOpenFileName("/dev/midi02", "All Files (*.*)", this));

	if (midi_port_name[0])
	{
		if (midi_port >= 0)
			close(midi_port);

		if ((midi_port = open(midi_port_name, O_WRONLY)) < 0)
			QMessageBox::information(this, strerror(errno), "Failure opening UA-100 Control Port", QMessageBox::Ok);
	}
}

void usbua100mix::slotFileExit()
{
	qApp->quit();
}

void usbua100mix::slotHelpAbout()
{
	QMessageBox::about(this,"About...", USBUA100MIX_ABOUT);
}

void usbua100mix::slotFullEffect()
{
	full_effect->show();
	insertion_effect->hide();
	system_effect_1->hide();
	system_effect_2->hide();
}

void usbua100mix::slotInsertionEffect()
{
	full_effect->hide();
	insertion_effect->show();
	system_effect_1->hide();
	system_effect_2->hide();
}

void usbua100mix::slotSystemEffect1()
{
	full_effect->hide();
	insertion_effect->hide();
	system_effect_1->show();
	system_effect_2->hide();
}

void usbua100mix::slotSystemEffect2()
{
	full_effect->hide();
	insertion_effect->hide();
	system_effect_1->hide();
	system_effect_2->show();
}

void usbua100mix::slotChangeFullEffect(int index)
{
	send_ua100_sysex_message(0x00404000, 04); // full effects mode

	full_effect->clear_effect();
	insertion_effect->clear_effect();
	system_effect_1->clear_effect();
	system_effect_2->clear_effect();

	full_effect->name_box->setCurrentItem(index);

	if (index == 0)
	{
		send_ua100_sysex_message(0x00404000, 03); // compact effects mode
		return;
	}

	else if (index == 1)
	{
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_HIGH_QUALITY_REVERB;
		full_effect->add_parameter(0, "Type", usbua100mix_effect_parameter_reverb, 3);
		full_effect->add_parameter(1, "Pre", usbua100mix_effect_parameter_delay4, 0x70);
		full_effect->add_parameter(2, "Time", usbua100mix_effect_parameter_reverb_time, 120);

		full_effect->add_parameter(3, "Damp", usbua100mix_effect_parameter_hfdamp, 0x68); // really -10 - 0?
		full_effect->add_parameter(4, "ER Del", usbua100mix_effect_parameter_delay4, 8);
		full_effect->add_parameter(5, "ER Mix", usbua100mix_effect_parameter_percent, 15);

		full_effect->add_parameter(6, "Diffuse", usbua100mix_effect_parameter_10, 9);
		full_effect->add_parameter(7, "Rev Lo", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		full_effect->add_parameter(8, "Rev Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);

		full_effect->add_parameter(11, "Lo Gain", usbua100mix_effect_parameter_plus_minus_12db, 0x45);
		full_effect->add_parameter(14, "Mid1 Gain", usbua100mix_effect_parameter_plus_minus_12db, 0x48);
		full_effect->add_parameter(17, "Mid2 Gain", usbua100mix_effect_parameter_plus_minus_12db, 0x38);

		full_effect->add_parameter(19, "Hi Gain", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		full_effect->add_parameter(10, "EQ Lo", usbua100mix_effect_parameter_low_eq_freq, 1);
		full_effect->add_parameter(13, "Mid1 Q", usbua100mix_effect_parameter_q, 0);

		full_effect->add_parameter(12, "Mid1 Freq", usbua100mix_effect_parameter_eq_freq, 0x48);
		full_effect->add_parameter(16, "Mid2 Q", usbua100mix_effect_parameter_q, 0);
		full_effect->add_parameter(15, "Mid2 Freq", usbua100mix_effect_parameter_eq_freq, 0x38);

		full_effect->add_parameter(18, "EQ Hi", usbua100mix_effect_parameter_hi_eq_freq, 1);
		full_effect->add_parameter(9, "Mix", usbua100mix_effect_parameter_percent, 32);
	}

	else if (index == 2)
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_MICROPHONE_SIMULATOR;

	else if (index == 3)
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_VOCODER;

	else if (index == 4)
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_VOCAL_MULTI;

	else if (index == 5)
		full_effect->effect_number = USBUA100MIX_EFFECT_FULL_GAME;

	send_ua100_sysex_message(0x00400100 + (full_effect->path << 8), full_effect->effect_number, 2);
	send_ua100_sysex_message(0x00404001 + full_effect->path, 1);
	channels[0]->slotModeChange(channels[0]->input_mode->currentItem()); // doesn't apply effect to input otherwise
}

void usbua100mix::slotChangeInsertionEffect(int index)
{
	if (full_effect->name_box->currentItem() > 0)
	{
		full_effect->clear_effect();
		send_ua100_sysex_message(0x00404000, 03); // compact effects mode
	}

	insertion_effect->clear_effect();
	insertion_effect->name_box->setCurrentItem(index);

	// The Parameter area for combo boxes is 2,27 to 294,200
	// Combo boxes: 2 across = 145 x 23; 3 across = 95 x 23
	if (index == 0)
		return;

	else if (index == 1)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_NOISE_SUPPRESSOR;
		insertion_effect->add_parameter(0, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 2)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEREO_EQ;
		insertion_effect->add_parameter(1, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x45);
		insertion_effect->add_parameter(6, "Mid1", usbua100mix_effect_parameter_plus_minus_12db, 0x48);
		insertion_effect->add_parameter(9, "Mid2", usbua100mix_effect_parameter_plus_minus_12db, 0x38);

		insertion_effect->add_parameter(3, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x34);
		insertion_effect->add_parameter(0, "Low", usbua100mix_effect_parameter_low_eq_freq, 1);
		insertion_effect->add_parameter(4, "Mid1", usbua100mix_effect_parameter_eq_freq, 72);

		insertion_effect->add_parameter(7, "Mid2", usbua100mix_effect_parameter_eq_freq, 56);
		insertion_effect->add_parameter(2, "Hi", usbua100mix_effect_parameter_hi_eq_freq, 1);
		insertion_effect->add_parameter(5, "Mid1 Q", usbua100mix_effect_parameter_q, 0);

		insertion_effect->add_parameter(8, "Mid2 Q", usbua100mix_effect_parameter_q, 0);
		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 0x7f);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 3)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_SPECTRUM;
		insertion_effect->add_parameter(0, "250", usbua100mix_effect_parameter_plus_minus_12db, 0x3c);
		insertion_effect->add_parameter(1, "500", usbua100mix_effect_parameter_plus_minus_12db, 0x41);
		insertion_effect->add_parameter(2, "1K", usbua100mix_effect_parameter_plus_minus_12db, 0x43);

		insertion_effect->add_parameter(3, "1.3K", usbua100mix_effect_parameter_plus_minus_12db, 0x46);
		insertion_effect->add_parameter(4, "2K", usbua100mix_effect_parameter_plus_minus_12db, 0x42);
		insertion_effect->add_parameter(5, "3.2", usbua100mix_effect_parameter_plus_minus_12db, 0x3f);

		insertion_effect->add_parameter(6, "4K", usbua100mix_effect_parameter_plus_minus_12db, 0x3c);
		insertion_effect->add_parameter(7, "8K", usbua100mix_effect_parameter_plus_minus_12db, 0x3b);
		insertion_effect->add_parameter(8, "Width", usbua100mix_effect_parameter_q, 2);

		insertion_effect->add_parameter(18, "Pan", usbua100mix_effect_parameter_pan, 64);
		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 4)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ENHANCER;
		insertion_effect->add_parameter(0, "Sens.", usbua100mix_effect_parameter_percent, 64);
		insertion_effect->add_parameter(1, "Mix", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 127);

		insertion_effect->add_parameter(16, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x43);
		insertion_effect->add_parameter(17, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 5)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_HUMANIZER;
		insertion_effect->add_parameter(0, "Drive", usbua100mix_effect_parameter_percent, 48);
		insertion_effect->add_parameter(1, "Dist", usbua100mix_effect_parameter_off_on, 1);
		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 127);

		insertion_effect->add_parameter(2, "Vowel", usbua100mix_effect_parameter_vowel, 64);
		insertion_effect->add_parameter(3, "Speed", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(18, "Pan", usbua100mix_effect_parameter_pan, 64);

		insertion_effect->add_parameter(16, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(17, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 6)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE;
		insertion_effect->add_parameter(0, "Drive", usbua100mix_effect_parameter_percent, 48);
		insertion_effect->add_parameter(1, "Amp", usbua100mix_effect_parameter_amp, 1);
		insertion_effect->add_parameter(2, "Amp", usbua100mix_effect_parameter_off_on, 1);

		insertion_effect->add_parameter(16, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(17, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(18, "Pan", usbua100mix_effect_parameter_pan, 64);

		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 7)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_DISTORTION;
		insertion_effect->add_parameter(0, "Drive", usbua100mix_effect_parameter_percent, 48);
		insertion_effect->add_parameter(1, "Amp", usbua100mix_effect_parameter_amp, 1);
		insertion_effect->add_parameter(2, "Amp", usbua100mix_effect_parameter_off_on, 1);

		insertion_effect->add_parameter(16, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(17, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(18, "Pan", usbua100mix_effect_parameter_pan, 64);

		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 8)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_PHASER;
		insertion_effect->add_parameter(0, "Freq", usbua100mix_effect_parameter_manual, 36);
		insertion_effect->add_parameter(1, "Rate", usbua100mix_effect_parameter_rate1, 16);
		insertion_effect->add_parameter(2, "Depth", usbua100mix_effect_parameter_percent, 64);

		insertion_effect->add_parameter(3, "Reso", usbua100mix_effect_parameter_percent, 16);
		insertion_effect->add_parameter(4, "Mix", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 104);

		insertion_effect->add_parameter(16, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(17, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 9)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_AUTO_WAH;
		insertion_effect->add_parameter(0, "Filter", usbua100mix_effect_parameter_wide_narrow, 1);
		insertion_effect->add_parameter(1, "Sens.", usbua100mix_effect_parameter_percent, 0);
		insertion_effect->add_parameter(2, "Freq", usbua100mix_effect_parameter_manual, 68);

		insertion_effect->add_parameter(3, "Peak", usbua100mix_effect_parameter_percent, 62);
		insertion_effect->add_parameter(4, "Rate", usbua100mix_effect_parameter_rate1, 16);
		insertion_effect->add_parameter(5, "Depth", usbua100mix_effect_parameter_percent, 64);

		insertion_effect->add_parameter(6, "", usbua100mix_effect_parameter_direction, 1);
		insertion_effect->add_parameter(16, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(17, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);

		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 96);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 10)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ROTARY;

	else if (index == 11)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEREO_FLANGER;

	else if (index == 12)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEP_FLANGER;

	else if (index == 13)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_TREMELO;

	else if (index == 14)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_AUTO_PAN;

	else if (index == 15)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_COMPRESSOR;

	else if (index == 16)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_LIMITER;

	else if (index == 17)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_HEXA_CHORUS;

	else if (index == 18)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_TREMELO_CHORUS;

	else if (index == 19)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEREO_CHORUS;

	else if (index == 20)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_SPACE_D_CHORUS;

	else if (index == 21)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3D_CHORUS;

	else if (index == 22)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_STEREO_DELAY;

	else if (index == 23)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_MODULATION_DELAY;

	else if (index == 24)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3_TAP_DELAY;

	else if (index == 25)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_4_TAP_DELAY;

	else if (index == 26)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_TIME_CONTROL_DELAY;

	else if (index == 27)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_REVERB;
		insertion_effect->add_parameter(0, "Type", usbua100mix_effect_parameter_reverb, 3);
		insertion_effect->add_parameter(1, "Pre Del", usbua100mix_effect_parameter_pre_delay_time, 112);
		insertion_effect->add_parameter(2, "Length", usbua100mix_effect_parameter_percent, 120);

		insertion_effect->add_parameter(3, "HF Cut", usbua100mix_effect_parameter_hfdamp, 0x67);
		insertion_effect->add_parameter(17, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(18, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);

		insertion_effect->add_parameter(16, "Pan", usbua100mix_effect_parameter_pan, 0x40);
		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 28)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_GATE_REVERB;

	else if (index == 29)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3D_DELAY;

	else if (index == 30)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_2_VOICE_PITCH;

	else if (index == 31)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_FEEDBACK_PITCH;

	else if (index == 32)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3D_AUTO;

	else if (index == 33)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_3D_MANUAL;

	else if (index == 34)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_LO_FI_1;

	else if (index == 35)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_LO_FI_2;
		insertion_effect->add_parameter(0, "Type", usbua100mix_effect_parameter_5, 1);
		insertion_effect->add_parameter(3, "Radio Tun", usbua100mix_effect_parameter_percent, 0);
		insertion_effect->add_parameter(4, "Radio Nz", usbua100mix_effect_parameter_percent, 64);

		insertion_effect->add_parameter(5, "Nz", usbua100mix_effect_parameter_noise_type, 1);
		insertion_effect->add_parameter(6, "Nz Cut", usbua100mix_effect_parameter_lpf, 127);
		insertion_effect->add_parameter(7, "Nz Lev", usbua100mix_effect_parameter_percent, 0);

		insertion_effect->add_parameter(8, "Disc", usbua100mix_effect_parameter_disc, 0);
		insertion_effect->add_parameter(9, "Disc Cut", usbua100mix_effect_parameter_lpf, 127);
		insertion_effect->add_parameter(10, "Disc Lev", usbua100mix_effect_parameter_percent, 0);

		insertion_effect->add_parameter(11, "Hum", usbua100mix_effect_parameter_hum, 0);
		insertion_effect->add_parameter(12, "Hum Cut", usbua100mix_effect_parameter_lpf, 127);
		insertion_effect->add_parameter(13, "Hum Lev", usbua100mix_effect_parameter_percent, 0);

		insertion_effect->add_parameter(15, "Mix", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(16, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(17, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);

		insertion_effect->add_parameter(14, "", usbua100mix_effect_parameter_mono, 1);
		insertion_effect->add_parameter(18, "Pan", usbua100mix_effect_parameter_pan, 0x40);
		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 127);

		insertion_effect->add_parameter(1, "Filter", usbua100mix_effect_parameter_filter_type, 1);
		insertion_effect->add_parameter(2, "Cutoff", usbua100mix_effect_parameter_cutoff, 32);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 36)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_CHORUS;

	else if (index == 37)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_FLANGER;

	else if (index == 38)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_DELAY;

	else if (index == 39)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_DISTORTION_CHORUS;

	else if (index == 40)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_DISTORTION_FLANGER;

	else if (index == 41)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_DISTORTION_DELAY;

	else if (index == 42)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ENHANCER_CHORUS;

	else if (index == 43)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ENHANCER_FLANGER;

	else if (index == 44)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ENHANCER_DELAY;

	else if (index == 45)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CHORUS_DELAY;

	else if (index == 46)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_FLANGER_DELAY;

	else if (index == 47)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CHORUS_FLANGER;

	else if (index == 48)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_ROTARY_MULTI;

	else if (index == 49)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI1;
		insertion_effect->add_parameter(0, "Attack", usbua100mix_effect_parameter_percent, 100);
		insertion_effect->add_parameter(1, "Sustn", usbua100mix_effect_parameter_percent, 80);
		insertion_effect->add_parameter(2, "Comp", usbua100mix_effect_parameter_percent, 100);

		insertion_effect->add_parameter(3, "Comp", usbua100mix_effect_parameter_off_on, 1);
		insertion_effect->add_parameter(4, "", usbua100mix_effect_parameter_overdrive_distortion, 0);
		insertion_effect->add_parameter(6, "", usbua100mix_effect_parameter_amp, 0);

		insertion_effect->add_parameter(7, "Amp", usbua100mix_effect_parameter_off_on, 1);
		insertion_effect->add_parameter(10, "OD", usbua100mix_effect_parameter_off_on, 1);
		insertion_effect->add_parameter(5, "Drive", usbua100mix_effect_parameter_percent, 80);

		insertion_effect->add_parameter(8, "OD Lo", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(9, "OD Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(19, "Level", usbua100mix_effect_parameter_percent, 110);

		insertion_effect->add_parameter(11, "", usbua100mix_effect_parameter_chorus_flanger, 1);
		insertion_effect->add_parameter(12, "Mod Rt", usbua100mix_effect_parameter_rate2, 8);
		insertion_effect->add_parameter(13, "Mod Dep", usbua100mix_effect_parameter_percent, 30);

		insertion_effect->add_parameter(14, "Chr FB", usbua100mix_effect_parameter_percent, 102);
		insertion_effect->add_parameter(15, "Chr Mx", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(16, "Delay", usbua100mix_effect_parameter_delay4, 0x3c);

		insertion_effect->add_parameter(17, "Del FB", usbua100mix_effect_parameter_percent, 34);
		insertion_effect->add_parameter(18, "Del Mx", usbua100mix_effect_parameter_percent, 15);
		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 50)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI2;

	else if (index == 51)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI3;

	else if (index == 52)
	{
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CLEAN_MULTI1;
		insertion_effect->add_parameter(4, "Low", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(7, "Mid", usbua100mix_effect_parameter_plus_minus_12db, 0x40);
		insertion_effect->add_parameter(8, "Hi", usbua100mix_effect_parameter_plus_minus_12db, 0x40);

		insertion_effect->add_parameter(5, "Mid", usbua100mix_effect_parameter_eq_freq, 0);
		insertion_effect->add_parameter(6, "Mid Q", usbua100mix_effect_parameter_q, 2);
		insertion_effect->add_parameter(3, "Comp", usbua100mix_effect_parameter_off_on, 1);

		insertion_effect->add_parameter(0, "Attck", usbua100mix_effect_parameter_percent, 50);
		insertion_effect->add_parameter(1, "Sust", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(2, "Comp", usbua100mix_effect_parameter_percent, 75);

		insertion_effect->add_parameter(9, "", usbua100mix_effect_parameter_chorus_flanger, 0);
		insertion_effect->add_parameter(10, "Mod Rate", usbua100mix_effect_parameter_rate2, 8);
		insertion_effect->add_parameter(11, "Mod Dep", usbua100mix_effect_parameter_percent, 30);

		insertion_effect->add_parameter(12, "Chr FB", usbua100mix_effect_parameter_percent, 102);
		insertion_effect->add_parameter(13, "Chr Mx", usbua100mix_effect_parameter_percent, 127);
		insertion_effect->add_parameter(14, "Delay", usbua100mix_effect_parameter_delay4, 0x3c);

		insertion_effect->add_parameter(15, "Del FB", usbua100mix_effect_parameter_percent, 34);
		insertion_effect->add_parameter(16, "Del Cut", usbua100mix_effect_parameter_hfdamp, 34);
		insertion_effect->add_parameter(17, "Del Mx", usbua100mix_effect_parameter_percent, 15);

		insertion_effect->add_parameter(34, "Gate", usbua100mix_effect_parameter_percent, 10);
	}

	else if (index == 53)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CLEAN_MULTI2;

	else if (index == 54)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_BASS_BULTI;

	else if (index == 55)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_PIANO_MULTI;

	else if (index == 56)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_KEYBOARD_MULTI;

	else if (index == 57)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CHORUS_DELAY_PARALLEL;

	else if (index == 58)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_FLANGER_DELAY_PARALLEL;

	else if (index == 59)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_CHORUS_FLANGER_PARALLEL;

	else if (index == 60)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_DISTORTION;

	else if (index == 61)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_ROTARY;

	else if (index == 62)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_PHASER;

	else if (index == 63)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_AUTOWAH;

	else if (index == 64)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_PHASER_ROTARY;

	else if (index == 65)
		insertion_effect->effect_number = USBUA100MIX_EFFECT_INSERTION_PHASER_AUTOWAH;

	send_ua100_sysex_message(0x00400100 + (insertion_effect->path << 8), insertion_effect->effect_number, 2);
	send_ua100_sysex_message(0x00404001 + insertion_effect->path, 1);
	channels[0]->slotModeChange(channels[0]->input_mode->currentItem()); // doesn't apply effect to input otherwise
}

void usbua100mix::slotChangeSystemEffect1(int index)
{
	if (full_effect->name_box->currentItem() > 0)
	{
		full_effect->clear_effect();
		send_ua100_sysex_message(0x00404000, 03); // compact effects mode
	}

	system_effect_1->clear_effect();
	system_effect_1->name_box->setCurrentItem(index);

	if (index == 0)
		return;

	else if (index == 1)
		system_effect_1->effect_number = USBUA100MIX_EFFECT_SYS1_DELAY;

	else if (index == 2)
		system_effect_1->effect_number = USBUA100MIX_EFFECT_SYS1_CHORUS;

	send_ua100_sysex_message(0x00400500, system_effect_1->effect_number, 2);
	send_ua100_sysex_message(0x00404005, 1);
	channels[0]->slotModeChange(channels[0]->input_mode->currentItem()); // doesn't apply effect to input otherwise
}

void usbua100mix::slotChangeSystemEffect2(int index)
{
	if (full_effect->name_box->currentItem() > 0)
	{
		full_effect->clear_effect();
		send_ua100_sysex_message(0x00404000, 03); // compact effects mode
	}

	system_effect_2->clear_effect();
	system_effect_2->name_box->setCurrentItem(index);

	if (index == 0)
		return;

	else if (index == 1)
		system_effect_2->effect_number = USBUA100MIX_EFFECT_SYS2_DELAY;

	else if (index == 2)
	{
		/***
		system_effect_2->effect_number = USBUA100MIX_EFFECT_SYS2_REVERB;
		system_effect_2->parameter[0] = new usbua100mix_effect_parameter(system_effect_2, 154, 2, 298, 25, 0, 3, "Room1", "Room2", "Stage1", "Stage2", "Hall1", "Hall2");
		system_effect_2->parameter[1] = new usbua100mix_effect_parameter(system_effect_2, 2, 25, 32, 78, 1, "Pre", 0, 127, 0x70);
		system_effect_2->parameter[2] = new usbua100mix_effect_parameter(system_effect_2, 34, 25, 64, 78, 2, "Time", 0, 127, 120);
		system_effect_2->parameter[3] = new usbua100mix_effect_parameter(system_effect_2, 66, 25, 96, 78, 3, "Damp", 0, 127, 0x67);
		system_effect_2->parameter[4] = new usbua100mix_effect_parameter(system_effect_2, 98, 25, 128, 78, 4, "Lo", 0x34, 0x4c, 0x40);
		system_effect_2->parameter[5] = new usbua100mix_effect_parameter(system_effect_2, 130, 25, 160, 78, 5, "Hi", 0x34, 0x4c, 0x40);
		system_effect_2->parameter[6] = new usbua100mix_effect_parameter(system_effect_2, 162, 25, 192, 78, 6, "Balance", 0, 127, 0x40);
		***/
	}

	send_ua100_sysex_message(0x00400600, system_effect_2->effect_number, 2);
	send_ua100_sysex_message(0x00404006, 1);
	channels[0]->slotModeChange(channels[0]->input_mode->currentItem()); // doesn't apply effect to input otherwise
}


/********************** Events ************************/

void usbua100mix::closeEvent(QCloseEvent *event)
{
	QWidget::closeEvent(event);
}

void usbua100mix::keyPressEvent(QKeyEvent *event)
{
	/* QMessageBox::information(this, "Keystrokes", "Feature Not Implemented", QMessageBox::Ok); */

	QMainWindow::keyPressEvent(event);
}


/****************************** Effect Base Widget **************************************/

usbua100mix_effect::usbua100mix_effect(usbua100mix *mixer, int path) : QWidget(mixer)
{
	move(302, 50);
	resize(296, 225);

	name_box = new QComboBox(this);
	name_box->move(2, 2);
	name_box->resize(196, 23);

	if (path >= 0)
	{
		this->path = path;
		path_box = NULL;
	}
	else
	{
		path_box = new QComboBox(this);
		path_box->move(200, 2);
		path_box->resize(94, 23);
		path_box->insertItem("Line/Mic 1");
		path_box->insertItem("Mic 2");
		path_box->insertItem("Wave 1");
		path_box->insertItem("Wave 2");
		path_box->insertItem("Sys 1");
		path_box->insertItem("Sys 2");
		path_box->setCurrentItem(0);
		path_box->setEditable(FALSE);
		connect(path_box, SIGNAL(activated(int)), this, SLOT(slotPathChanged(int)));
		this->path = 0;
	}

	setBackgroundMode(QWidget::PaletteMid);

	this->effect_number = 0;
	this->mixer = mixer;
	memset(parameter, 0, sizeof(parameter));
}

void usbua100mix_effect::add_parameter(int parameter_number, char *prefix, char **values, int default_value)
{
	int left_x = 2 + (99 * (number_of_parameters() % 3));
	int top_y = 27 + (23 * (number_of_parameters() / 3));

	parameter[parameter_number] = new QComboBox(this);
	parameter[parameter_number]->move(left_x, top_y);
	parameter[parameter_number]->resize(97, 25);

	for (int entry = 0; values[entry] && (entry < 128); ++entry)
	{
		char composite[64];
		sprintf(composite, "%s %s", prefix, values[entry]);
		parameter[parameter_number]->insertItem(composite);
	}

	parameter[parameter_number]->setCurrentItem(default_value);
	parameter[parameter_number]->setEditable(FALSE);
	mixer->send_ua100_sysex_message(0x00400103 + (path << 8) + parameter_number, default_value);

	connect(parameter[parameter_number], SIGNAL(activated(int)), this, SLOT(slotParameterChanged(int)));
	parameter[parameter_number]->show();
}

void usbua100mix_effect::clear_effect()
{
	mixer->send_ua100_sysex_message(0x00404001 + path, 0);

	for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
		if (parameter[scan])
		{
			delete parameter[scan];
			parameter[scan] = NULL;
		}

	name_box->setCurrentItem(0);
}

void usbua100mix_effect::slotPathChanged(int new_path)
{
	clear_effect();
	path = new_path;
	mixer->send_ua100_sysex_message(0x00404001 + path, 1);
}

void usbua100mix_effect::slotParameterChanged(int value)
{
	QComboBox *parameter_box = (QComboBox*) sender();
	int parameter_number = 0;

	while ((parameter_number < USBUA100MIX_EFFECT_PARAMETERS) && (parameter[parameter_number] != parameter_box))
		++parameter_number;

	if (mixer && (parameter_number < USBUA100MIX_EFFECT_PARAMETERS))
		mixer->send_ua100_sysex_message(0x00400103 + (path << 8) + parameter_number, value);
}

/********************** Main Program Entry Function ************************/

int main(int argc, char *argv[])
{
	usbua100mix_application application(argc, argv);
	application.connect(&application, SIGNAL(lastWindowClosed()), &application, SLOT(quit()));

	char *load_file_name = NULL;
	if (argc > 1)
		load_file_name = argv[1];

	application.mixer = new usbua100mix(load_file_name);

	application.mixer->show();

	return application.exec();
}
