/***************************************************************************
	usbua100mix.h  -  A Linux Mixer/Control Panel for the Roland UA-100
                             -------------------
    begin                : Thu June 6, 2001
    copyright            : (C) 2001 by Michael Minn
    email                : michael@michaelminn.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

***************************************************************************/

#ifndef USBUA100MIX_H
#define USBUA100MIX_H

#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <qapplication.h>
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qpopupmenu.h>
#include <qaccel.h>
#include <qmenubar.h>
#include <qapp.h>
#include <qslider.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qprogressbar.h>
#include <qcombobox.h>
#include <qtooltip.h>

#define USBUA100MIX_VERSION "USBUA100MIX v2001.12.27"
#define USBUA100MIX_ABOUT USBUA100MIX_VERSION "\n(c) 2001 by Michael Minn"
#define USBUA100MIX_CHANNELS 5
#define USBUA100MIX_EFFECT_PARAMETERS 40

#define USBUA100MIX_FILE_OPEN               0x3010
#define USBUA100MIX_FILE_SAVE               0x3020
#define USBUA100MIX_FILE_SAVE_AS            0x3030
#define USBUA100MIX_FILE_MIDI_PORT          0x3040
#define USBUA100MIX_FILE_EXIT               0x3050

#define USBUA100MIX_EFFECT_NONE 0x2000
#define USBUA100MIX_EFFECT_SYS1_NONE 0x2010
#define USBUA100MIX_EFFECT_SYS2_NONE 0x2020
#define USBUA100MIX_EFFECT_FULL_NONE 0x2030
#define USBUA100MIX_EFFECT_INSERTION_NONE 0x2040

#define USBUA100MIX_HELP_ABOUT              0x4000

#define USBUA100MIX_EFFECT_SYS1_DELAY 0x0021
#define USBUA100MIX_EFFECT_SYS1_CHORUS 0x0022

#define USBUA100MIX_EFFECT_SYS2_DELAY 0x0031
#define USBUA100MIX_EFFECT_SYS2_REVERB 0x0032

#define USBUA100MIX_EFFECT_FULL_HIGH_QUALITY_REVERB 0x0011
#define USBUA100MIX_EFFECT_FULL_MICROPHONE_SIMULATOR 0x0012
#define USBUA100MIX_EFFECT_FULL_VOCODER 0x0013
#define USBUA100MIX_EFFECT_FULL_VOCAL_MULTI 0x0014
#define USBUA100MIX_EFFECT_FULL_GAME 0x0016

#define USBUA100MIX_EFFECT_INSERTION_NOISE_SUPPRESSOR 0x0000
#define USBUA100MIX_EFFECT_INSERTION_STEREO_EQ 0x0100
#define USBUA100MIX_EFFECT_INSERTION_SPECTRUM 0x0101
#define USBUA100MIX_EFFECT_INSERTION_ENHANCER 0x0102
#define USBUA100MIX_EFFECT_INSERTION_HUMANIZER 0x0103
#define USBUA100MIX_EFFECT_INSERTION_OVERDRIVE 0x0110
#define USBUA100MIX_EFFECT_INSERTION_DISTORTION 0x0111
#define USBUA100MIX_EFFECT_INSERTION_PHASER 0x0120
#define USBUA100MIX_EFFECT_INSERTION_AUTO_WAH 0x0121
#define USBUA100MIX_EFFECT_INSERTION_ROTARY 0x0122
#define USBUA100MIX_EFFECT_INSERTION_STEREO_FLANGER 0x0123
#define USBUA100MIX_EFFECT_INSERTION_STEP_FLANGER 0x0124
#define USBUA100MIX_EFFECT_INSERTION_TREMELO 0x0125
#define USBUA100MIX_EFFECT_INSERTION_AUTO_PAN 0x0126
#define USBUA100MIX_EFFECT_INSERTION_COMPRESSOR 0x0130
#define USBUA100MIX_EFFECT_INSERTION_LIMITER 0x0131
#define USBUA100MIX_EFFECT_INSERTION_HEXA_CHORUS 0x0140
#define USBUA100MIX_EFFECT_INSERTION_TREMELO_CHORUS 0x0141
#define USBUA100MIX_EFFECT_INSERTION_STEREO_CHORUS 0x0142
#define USBUA100MIX_EFFECT_INSERTION_SPACE_D_CHORUS 0x0143
#define USBUA100MIX_EFFECT_INSERTION_3D_CHORUS 0x0144
#define USBUA100MIX_EFFECT_INSERTION_STEREO_DELAY 0x0150
#define USBUA100MIX_EFFECT_INSERTION_MODULATION_DELAY 0x0151
#define USBUA100MIX_EFFECT_INSERTION_3_TAP_DELAY 0x0152
#define USBUA100MIX_EFFECT_INSERTION_4_TAP_DELAY 0x0153
#define USBUA100MIX_EFFECT_INSERTION_TIME_CONTROL_DELAY 0x0154
#define USBUA100MIX_EFFECT_INSERTION_REVERB 0x0155
#define USBUA100MIX_EFFECT_INSERTION_GATE_REVERB 0x0156
#define USBUA100MIX_EFFECT_INSERTION_3D_DELAY 0x0157
#define USBUA100MIX_EFFECT_INSERTION_2_VOICE_PITCH 0x0160
#define USBUA100MIX_EFFECT_INSERTION_FEEDBACK_PITCH 0x0161
#define USBUA100MIX_EFFECT_INSERTION_3D_AUTO 0x0170
#define USBUA100MIX_EFFECT_INSERTION_3D_MANUAL 0x0171
#define USBUA100MIX_EFFECT_INSERTION_LO_FI_1 0x0172
#define USBUA100MIX_EFFECT_INSERTION_LO_FI_2 0x0173
#define USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_CHORUS 0x0200
#define USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_FLANGER 0x0201
#define USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_DELAY 0x0202
#define USBUA100MIX_EFFECT_INSERTION_DISTORTION_CHORUS 0x0203
#define USBUA100MIX_EFFECT_INSERTION_DISTORTION_FLANGER 0x0204
#define USBUA100MIX_EFFECT_INSERTION_DISTORTION_DELAY 0x0205
#define USBUA100MIX_EFFECT_INSERTION_ENHANCER_CHORUS 0x0206
#define USBUA100MIX_EFFECT_INSERTION_ENHANCER_FLANGER 0x0207
#define USBUA100MIX_EFFECT_INSERTION_ENHANCER_DELAY 0x0208
#define USBUA100MIX_EFFECT_INSERTION_CHORUS_DELAY 0x0209
#define USBUA100MIX_EFFECT_INSERTION_FLANGER_DELAY 0x020a
#define USBUA100MIX_EFFECT_INSERTION_CHORUS_FLANGER 0x020b
#define USBUA100MIX_EFFECT_INSERTION_ROTARY_MULTI 0x0300
#define USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI1 0x0400
#define USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI2 0x0401
#define USBUA100MIX_EFFECT_INSERTION_GUITAR_MULTI3 0x0402
#define USBUA100MIX_EFFECT_INSERTION_CLEAN_MULTI1 0x0403
#define USBUA100MIX_EFFECT_INSERTION_CLEAN_MULTI2 0x0404
#define USBUA100MIX_EFFECT_INSERTION_BASS_BULTI 0x0405
#define USBUA100MIX_EFFECT_INSERTION_PIANO_MULTI 0x0406
#define USBUA100MIX_EFFECT_INSERTION_KEYBOARD_MULTI 0x0500
#define USBUA100MIX_EFFECT_INSERTION_CHORUS_DELAY_PARALLEL 0x1100
#define USBUA100MIX_EFFECT_INSERTION_FLANGER_DELAY_PARALLEL 0x1101
#define USBUA100MIX_EFFECT_INSERTION_CHORUS_FLANGER_PARALLEL 0x1102
#define USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_DISTORTION 0x1103
#define USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_ROTARY 0x1104
#define USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_PHASER 0x1105
#define USBUA100MIX_EFFECT_INSERTION_OVERDRIVE_AUTOWAH 0x1106
#define USBUA100MIX_EFFECT_INSERTION_PHASER_ROTARY 0x1107
#define USBUA100MIX_EFFECT_INSERTION_PHASER_AUTOWAH 0x1108

class usbua100mix;

class usbua100mix_effect : public QWidget
{
	Q_OBJECT
	public:
	usbua100mix_effect(usbua100mix *mixer, int path = -1);
	int path;
	int effect_number;
	usbua100mix *mixer;
	QComboBox *name_box;
	QComboBox *path_box;
	QComboBox *parameter[USBUA100MIX_EFFECT_PARAMETERS];

	void clear_effect();
	void add_parameter(int parameter_number, char *prefix, char **values, int default_value);

	int number_of_parameters()
	{
		int count = 0;
		for (int scan = 0; scan < USBUA100MIX_EFFECT_PARAMETERS; ++scan)
			if (parameter[scan])
				++count;
		return count;
	}

	public slots:
	void slotPathChanged(int new_path);
	void slotParameterChanged(int value);
};

/*************************************************************************************/

class usbua100mix_channel : public QWidget
{
	Q_OBJECT
	public:
	usbua100mix_channel(usbua100mix *mixer, int channel, char *name);

	int channel;
	QComboBox *input_mode;
	QLineEdit *name_box;
	QSlider *send1;
	QSlider *send2;
	QSlider *fader;
	QPushButton *record;
	QPushButton *output;
	QPushButton *mute;

	public slots:
	void slotFader(int value);
	void slotSend1(int value);
	void slotSend2(int value);
	void slotRecord();
	void slotOutput();
	void slotMute();
	void slotModeChange(int mode);
};

class usbua100mix : public QMainWindow
{
	Q_OBJECT
	public:
	usbua100mix(const char *file_name);
	~usbua100mix() {}
	int send_ua100_sysex_message(unsigned int address, unsigned int value, int data_bytes = 1);

	int midi_port;
	char midi_port_name[256];

	QPopupMenu *fileMenu;
	QPopupMenu *helpMenu;

	usbua100mix_channel *channels [USBUA100MIX_CHANNELS];
	QButtonGroup *record_button_group;
	QButtonGroup *output_button_group;

	QPushButton *full_effect_button;
	QPushButton *insertion_effect_button;
	QPushButton *system_effect_1_button;
	QPushButton *system_effect_2_button;
	QButtonGroup *effect_button_group;

	usbua100mix_effect *full_effect;
	usbua100mix_effect *insertion_effect;
	usbua100mix_effect *system_effect_1;
	usbua100mix_effect *system_effect_2;

	protected:
	virtual void closeEvent(QCloseEvent *event);
	virtual void keyPressEvent(QKeyEvent *event);

	int save_mixer_settings(const char *file_name);
	int load_mixer_settings(const char *file_name);

	public slots:
	void slotFileOpen();
	void slotFileSave();
	void slotFileSaveAs();
	void slotFileMIDIPort();
	void slotFileExit();

	void slotHelpAbout();

	void slotFullEffect();
	void slotInsertionEffect();
	void slotSystemEffect1();
	void slotSystemEffect2();

	void slotChangeFullEffect(int);
	void slotChangeInsertionEffect(int);
	void slotChangeSystemEffect1(int);
	void slotChangeSystemEffect2(int);
};

class usbua100mix_application : public QApplication
{
	public:
	usbua100mix_application(int argc, char** argv) : QApplication(argc, argv) { mixer = NULL; };
	usbua100mix *mixer;
};


#endif
